const Square = ({onClick, content}) => (
  <button className='square' onClick={onClick}>{content}</button>
)

const Row = ({onMove, squares, startIndex}) => (
  <div className='row'>
    <Square onClick={() => onMove(startIndex)}
      content={squares[startIndex    ]} />
    <Square onClick={() => onMove(startIndex + 1)}
      content={squares[startIndex + 1]} />
    <Square onClick={() => onMove(startIndex + 2)}
      content={squares[startIndex + 2]} />
  </div>
)

const Board = ({onMove, squares}) => (
  <div className='board'>
    <Row onMove={onMove} squares={squares} startIndex={0} />
    <Row onMove={onMove} squares={squares} startIndex={3} />
    <Row onMove={onMove} squares={squares} startIndex={6} />
  </div>
)

const Status = ({xIsNext, winner}) => {
  const text = winner === null ? 'Tie game :-/'
        : winner !== undefined ? `${winner} wins!`
        : `It is ${xIsNext ? 'X' : 'O'}'s turn.`
  return (
    <div className='status'>
      {text}
    </div>
  )
}

class App extends React.Component {
  constructor(props) {
    super(props)
    this.state = {
      xIsNext: true,
      squares: [
        'X', 'O', 'X',
        null, 'O', null,
        null, null, null,
      ]
    }
  }

  onMove(index) {
    const newSquares = this.state.squares.slice()
    newSquares[index] = this.state.xIsNext ? 'X' : 'O'
    this.setState({
      squares: newSquares,
      xIsNext: !this.state.xIsNext,
    })
  }

  winner() {
    const lines = [
      [0, 1, 2],
      [3, 4, 5],
      [6, 7 ,8],
      [0, 3, 6],
      [1, 4, 7],
      [2, 5, 8],
      [0, 4, 8],
      [2, 4, 6],
    ]
    const squares = this.state.squares
    for (let i=0; i<lines.length; i++) {
      const [a, b, c] = lines[i]
      if (squares[a] && squares[a] == squares[b] && squares[a] == squares[c]) {
        return squares[a]
      }
    }
    if (squares.indexOf(null) === -1) return null // tie game
    return undefined
  }

  render() {
    return (
      <div>
        <Board onMove={(index) => this.onMove(index)} squares={this.state.squares} />
        <Status xIsNext={this.state.xIsNext} winner={this.winner()} />
      </div>
    )
  }
}

ReactDOM.render(
  <App />,
  document.getElementById('root')
)
